;/******************************/;
;/* Example For : JRHC08JL3 V2 */;
;/* Controller  : 68HC908JL3   */;
;/* Assembler   : CASM08Z.EXE  */;
;/* Demo DAC Gen."Sine" Signal */;
;/* DAC0..7 = Sine Wave Signal */;
;/******************************/;
;
;/*************************/;
;/* Register HC08 Equates */;
;/*************************/;
;
PORTA       EQU     $0000               ; Port-A Data Register
PORTB       EQU     $0001               ; Port-B Data Register
PORTD       EQU     $0003               ; Port-D Data Regsiter
DDRA        EQU     $0004               ; Port-A Direction Register
DDRB        EQU     $0005               ; Port-B Direction Register
DDRD        EQU     $0007               ; Port-D Direction Register
PDCR        EQU     $000A               ; Port-D Control Register
PTAPUE      EQU     $000D               ; Port-A Pull-Up Enable
KBSCR       EQU     $001A               ; Keyboard Status Control
KBIER       EQU     $001B               ; Keyboard Interrupt Enable
INTSCR      EQU     $001D               ; IRQ Status Control
CONFIG2     EQU     $001E               ; Config Register2
CONFIG1     EQU     $001F               ; Config Register1
TSC         EQU     $0020               ; TIM Status & Control
TCNTH       EQU     $0021               ; TIM Counter High
TCNTL       EQU     $0022               ; TIM Counter Low
TMODH       EQU     $0023               ; TIM Counter Modulo High
TMODL       EQU     $0024               ; TIM Counter Modulo Low
TSC0        EQU     $0025               ; TIM CH0 Status & Control
TCH0H       EQU     $0026               ; TIM CH0 High
TCH0L       EQU     $0027               ; TIM CH0 Low
TSC1        EQU     $0028               ; TIM CH1 Status & Control
TCH1H       EQU     $0029               ; TIM CH1 High
TCH1L       EQU     $002A               ; TIM CH1 Low
ADSCR       EQU     $003C               ; ADC Status & Control
ADR         EQU     $003D               ; ADC Data Read
ADICLK      EQU     $003E               ; ADC Input Clock
BSR         EQU     $FE00               ; Break Status Register
RSR         EQU     $FE01               ; Reset Status Register
BFCR        EQU     $FE03               ; Break Flag Control
INT1        EQU     $FE04               ; Interrupt Status1
INT2        EQU     $FE05               ; Interrupt Status2
INT3        EQU     $FE06               ; Interrupt Status3
FLCR        EQU     $FE08               ; Flash Control Register
FLBPR       EQU     $FE09               ; Flash Block Protect
BRKH        EQU     $FE0C               ; Break Address High
BRKL        EQU     $FE0D               ; Break Address Low
BRKSCR      EQU     $FE0E               ; Break Status & Control
COPCTL      EQU     $FFFF               ; Reset COP
;/* End Equate I/O Register of HC08 */;
;
;/********************/;
;/* Register Equates */;
;/********************/;
;
PORT_LCD    EQU     PORTD           ; PD0,PD1,PD2 Control 74HC595
LATCH_LCD   EQU     00H             ; Bit0 of Port Control LCD
SHIFT_LCD   EQU     01H             ; Bit1 of Port Control LCD
DATA_LCD    EQU     02H             ; Bit2 of Port Control LCD
PORT_DAC    EQU     PORTD           ; PD3,PD4,PD5 Control DAC
ENA_DAC     EQU     03H             ; Bit3 "0" = Enable CS# For DAC
DIN_DAC     EQU     04H             ; Bit4 = Data Input For DAC
SCK_DAC     EQU     05H             ; Bit5 = Serial Clock For DAC
PORT_SEL    EQU     PORTA           ; PA4,PA5 Select CS#0..3 For DAC
DAC_SEL0    EQU     04H             ; Bit4 = A-Select 74HCT139
DAC_SEL1    EQU     05H             ; Bit5 = B-Select 74HCT139
;
RS_LCD      EQU     02H             ; Bit2 of 74HC595
EN_LCD      EQU     03H             ; Bit3 of 74HC595
D4_LCD      EQU     04H             ; Bit4 of 74HC595
D5_LCD      EQU     05H             ; Bit5 of 74HC595
D6_LCD      EQU     06H             ; Bit6 of 74HC595
D7_LCD      EQU     07H             ; Bit7 of 74HC595

            ORG     80H
LCD_SIGN    DS      1               ; LCD Data Signal Buffer
SINE_CNT    DS      1               ; Sine Step Count
SINE_BUF    DS      2               ; Sine Wave Buffer
$BASE       10T                     ; Defualt Number to Decimal

            ;/****************************/;
            ;/* 68HC908JL3 Port I/O Test */;
            ;/****************************/;
            ;
            ORG     $EC00                   ; Start Main (68HC908JL3)
            ;
RESET:      BSET    0,CONFIG1               ; Disable COP Watch-Dog
            MOV     #0,LCD_SIGN             ; Clear Signal Buffer
            LDA     #%11111111              ; Port PD7..0 = Output
            STA     DDRD
            LDA     #%00110000              ; Port PA5..4 = Output,PA3..0 = Input
            STA     DDRA
            LDA     #%00001111              ; PA3..0 Input Pull-Up Enable
            STA     PTAPUE
            ;
            BCLR    DATA_LCD,PORT_LCD       ; Initial 74HC595
            BCLR    LATCH_LCD,PORT_LCD
            BCLR    SHIFT_LCD,PORT_LCD
            ;
            BCLR    SCK_DAC,PORT_DAC        ; Initial DAC Control
            BCLR    DIN_DAC,PORT_DAC
            BSET    ENA_DAC,PORT_DAC        ; Disable CS# Output
            ;
NEW_SINE:   MOV     #0,SINE_CNT             ; Start New Sine
OUT_SINE:   JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #0                      ; Active DAC-0
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #1                      ; Active DAC-1
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #2                      ; Active DAC-2
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #3                      ; Active DAC-3
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #4                      ; Active DAC-4
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #5                      ; Active DAC-5
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #6                      ; Active DAC-6
            JSR     OUT_DACX                ; Out DAC = H:X
            JSR     GET_SINE                ; Get Function Sine
            LDHX    SINE_BUF
            LDA     #7                      ; Active DAC-7
            JSR     OUT_DACX                ; Out DAC = H:X
            ;
            LDA     SINE_CNT                ; Next Sine Step
            INCA
            STA     SINE_CNT
            CBEQA   #72,NEW_SINE            ; 360 / 5 = 72 Step
            BRA     OUT_SINE

;/***********************/;
;/* Get Sine From Table */;
;/* Input  : SINE_CNT   */;
;/* Output : SINE_BUF   */;
;/***********************/;
;
GET_SINE:   LDHX    #TAB_SINE               ; Table Sine Function
            PSHH                            ; Save H
            PSHX                            ; Save X
            LDA     SINE_CNT
            LSLA                            ; SINE_CNT x 2
            ADD     1,SP                    ; SINE_CNT + X
            STA     1,SP
            CLRA
            ADC     2,SP                    ; H+ 0 + CY
            STA     2,SP
            PULX                            ; Restore X
            PULH                            ; Restore H
            ;
            LDA     0,X
            STA     SINE_BUF+0              ; High Byte
            AIX     #1
            LDA     0,X
            STA     SINE_BUF+1              ; Low Byte
            RTS

;/*********************/;
;/* Table Sine 0..360 */;
;/* (5 Degree / Step) */;
;/*  Sine(n)*512+512  */;
;/*********************/;
;
TAB_SINE:   DW      511,556,600,643         ; Sine(00..15)
            DW      686,727,767,804         ; Sine(20..35)
            DW      839,872,902,926         ; Sine(40..55)
            DW      954,974,991,1005        ; Sine(60..75)
            DW      1014,1020,1023,1020     ; Sine(80..95)
            DW      1014,1005,991,974       ; Sine(100..115)
            DW      954,926,902,872         ; Sine(120..135)
            DW      839,804,767,727         ; Sine(140..155)
            DW      686,643,600,556         ; Sine(160..175)
            DW      511,466,422,379         ; Sine(180..195)
            DW      336,295,256,218         ; Sine(200..215)
            DW      183,150,120,92          ; Sine(220..235)
            DW      68,48,31,17             ; Sine(240..255)
            DW      8,2,0,2                 ; Sine(260..275)
            DW      8,17,31,48              ; Sine(280..295)
            DW      68,92,120,150           ; Sine(300..315)
            DW      183,218,256,295         ; Sine(320..335)
            DW      336,379,422,466         ; Sine(340..355)

;/***************************************/;
;/* Active Output Voltage of DAC Device */;
;/* Input  : A  (Channel 0..7)          */;
;/*        : H:X (Data 10Bit 000..3FFH) */;
;/* Remark : Ch0 = Ch-A of DAC Divice-0 */;
;/*        : Ch1 = Ch-B of DAC Divice-0 */;
;/*        : Ch2 = Ch-A of DAC Divice-1 */;
;/*        : Ch3 = Ch-B of DAC Divice-1 */;
;/*        : Ch4 = Ch-A of DAC Divice-2 */;
;/*        : Ch5 = Ch-B of DAC Dicice-2 */;
;/*        : Ch6 = Ch-A of DAC Divice-3 */;
;/*        : Ch7 = Ch-B of DAC Divice-3 */;
;/***************************************/;
;
OUT_DACX:   PSHH                            ; Save Register
            PSHX
            PSHA
            TPA                             ; CCR -> A
            PSHA
            ;
            LDA     2,SP                    ; Get Channel DAC (A)
            CBEQA   #0,OUT_DAC0             ; Active Ch-0
            CBEQA   #1,OUT_DAC1             ; Active Ch-1
            CBEQA   #2,OUT_DAC2             ; Active Ch-2
            CBEQA   #3,OUT_DAC3             ; Active Ch-3
            CBEQA   #4,OUT_DAC4             ; Active Ch-4
            CBEQA   #5,OUT_DAC5             ; Active Ch-5
            CBEQA   #6,OUT_DAC6             ; Active Ch-6
            CBEQA   #7,OUT_DAC7             ; Active Ch-7
            JMP     EXIT_DAC                ; Channel Error
            ;
            ;/* Convert Input to Control Word
            ;/* -> User Input 3 Bute HEX
            ;/*        A      :      H     :     X
            ;/*     ?????XXX  :  ??????98  :  76543210
            ;/*     ??? = Don't Care
            ;/*     XXX = Channel 0..7
            ;/*     0,1,2,3,4,5,6,7,8,9 = 10Bit Data
            ;/* -> Control Word For DAC Device
            ;/*    DAC-A : 10019876 543210??
            ;/*    DAC-B : 10109876 543210??
            ;
OUT_DAC0:   BCLR    DAC_SEL0,PORT_SEL       ; Active Device-0
            BCLR    DAC_SEL1,PORT_SEL
            JSR     OUTA_DAC                ; Initial Control Word DAC-A
            BRA     SHF_DACX
            ;
OUT_DAC1:   BCLR    DAC_SEL0,PORT_SEL       ; Active Device-0
            BCLR    DAC_SEL1,PORT_SEL
            JSR     OUTB_DAC                ; Initial Control Word DAC-B
            BRA     SHF_DACX
            ;
OUT_DAC2:   BSET    DAC_SEL0,PORT_SEL       ; Active Device-1
            BCLR    DAC_SEL1,PORT_SEL
            JSR     OUTA_DAC                ; Initial Control Word DAC-A
            BRA     SHF_DACX
            ;
OUT_DAC3:   BSET    DAC_SEL0,PORT_SEL       ; Active Device-1
            BCLR    DAC_SEL1,PORT_SEL
            JSR     OUTB_DAC                ; Initial Control Word DAC-B
            BRA     SHF_DACX
            ;
OUT_DAC4:   BCLR    DAC_SEL0,PORT_SEL       ; Active Device-2
            BSET    DAC_SEL1,PORT_SEL
            JSR     OUTA_DAC                ; Initial Control Word DAC-A
            BRA     SHF_DACX
            ;
OUT_DAC5:   BCLR    DAC_SEL0,PORT_SEL       ; Active Device-2
            BSET    DAC_SEL1,PORT_SEL
            JSR     OUTB_DAC                ; Initial Control Word DAC-B
            BRA     SHF_DACX
            ;
OUT_DAC6:   BSET    DAC_SEL0,PORT_SEL       ; Active Device-3
            BSET    DAC_SEL1,PORT_SEL
            JSR     OUTA_DAC                ; Initial Control Word DAC-A
            BRA     SHF_DACX
            ;
OUT_DAC7:   BSET    DAC_SEL0,PORT_SEL       ; Active Device-3
            BSET    DAC_SEL1,PORT_SEL
            JSR     OUTB_DAC                ; Initial Control Word DAC-B
            ;
            ;/* Start Shift-Out Control & Data to DAC */;
            ;/* A = XXXX9876
            ;/* X = 543210??
SHF_DACX:   PSHX                            ; Save X (543210??)
            BCLR    DIN_DAC,PORT_DAC        ; Initial DAC Control
            BCLR    SCK_DAC,PORT_DAC
            BCLR    ENA_DAC,PORT_DAC        ; Enable CS# Output
            LDX     #8                      ; 8 Bit Shift-Out
            ;
SHF_DAC0:   ROLA                            ; Shift D7..D0 (D7 First)
            BCS     SHF_DAC1                ; Jump if Data = "1"
            BCLR    DIN_DAC,PORT_DAC
            BRA     SHF_DAC2
            ;
SHF_DAC1:   BSET    DIN_DAC,PORT_DAC
SHF_DAC2:   NOP
            BSET    SCK_DAC,PORT_DAC      ; Shift Bit Data to DAC
            NOP
            NOP
            BCLR    SCK_DAC,PORT_DAC
            DBNZX   SHF_DAC0
            ;
            PULA                            ; Restore X to A(543210??)
            LDX     #8                      ; 8 Bit Shift-Out
            ;
SHF_DAC3:   ROLA                            ; Shift D7..D0 (D7 First)
            BCS     SHF_DAC4                ; Jump if Data = "1"
            BCLR    DIN_DAC,PORT_DAC
            BRA     SHF_DAC5
            ;
SHF_DAC4:   BSET    DIN_DAC,PORT_DAC
SHF_DAC5:   NOP
            BSET    SCK_DAC,PORT_DAC      ; Shift Bit Data to DAC
            NOP
            NOP
            BCLR    SCK_DAC,PORT_DAC
            DBNZX   SHF_DAC3
            ;
            BCLR    DIN_DAC,PORT_DAC
            BSET    ENA_DAC,PORT_DAC        ; Disable CS# Output
            ;
EXIT_DAC:   PULA                            ; Restore Register
            TAP                             ; A -> CCR
            PULA
            PULX
            PULH
            RTS

;/**************************/;
;/* Active DAC:A of Device */;
;/* Output : A (10019876)  */;
;/*        : X (543210??)  */;
;/**************************/;
;
OUTA_DAC:   LDA     6,SP                    ; A = ??????98 (H)
            AND     #%00000011              ; A = 00000098
            ORA     #%00100100              ; A = 00100198 (1001 = DAC-A of Device)
            LDX     5,SP                    ; X = Low Byte Data (X=76543210)
            LSLX                            ; C <- X(7..0) <- 0 (X=65432100)
            ROLA                            ; A(7..0) <- C      (A=01001987)
            LSLX                            ; C <- X(7..0) <- 0 (X=54321000)
            ROLA                            ; A(7..0) <- C      (A=10019876)
            RTS

;/**************************/;
;/* Active DAC:B of Device */;
;/* Output : A (10109876)  */;
;/*        : X (543210??)  */;
;/**************************/;
;
OUTB_DAC:   LDA     6,SP                    ; A = ??????98 (H)
            AND     #%00000011              ; A = 00000098
            ORA     #%00101000              ; A = 00101098 (1010 = DAC-B of Device)
            LDX     5,SP                    ; X = Low Byte Data (X=76543210)
            LSLX                            ; C <- X(7..0) <- 0 (X=65432100)
            ROLA                            ; A(7..0) <- C      (A=01010987)
            LSLX                            ; C <- X(7..0) <- 0 (X=54321000)
            ROLA                            ; A(7..0) <- C      (A=10109876)
            RTS

            ;/***************/;
            ;/* User Vector */;
            ;/***************/;
            ;
            ORG     $FFF6               ; Security Code
            DB      'JR08JLV2'
            ORG     $FFFE               ; Reset Vector
            DW      RESET

            END


