;/******************************/;
;/* Example For : JRHC08JL3 V1 */;
;/* Controller  : 68HC908JL3   */;
;/* Assembler   : CASM08Z.EXE  */;
;/* Demo For Buffer PWM Signal */;
;/* Use CH0 & CH1 Generate PWM */;
;/* Signal Output = TCH0(PTD4) */;
;/******************************/;
;
;/*************************/;
;/* Register HC08 Equates */;
;/*************************/;
;
PORTA       EQU     $0000                   ; Port-A Data Register
PORTB       EQU     $0001                   ; Port-B Data Register
PORTD       EQU     $0003                   ; Port-D Data Regsiter
DDRA        EQU     $0004                   ; Port-A Direction Register
DDRB        EQU     $0005                   ; Port-B Direction Register
DDRD        EQU     $0007                   ; Port-D Direction Register
PDCR        EQU     $000A                   ; Port-D Control Register
PTAPUE      EQU     $000D                   ; Port-A Pull-Up Enable
KBSCR       EQU     $001A                   ; Keyboard Status Control
KBIER       EQU     $001B                   ; Keyboard Interrupt Enable
INTSCR      EQU     $001D                   ; IRQ Status Control
CONFIG2     EQU     $001E                   ; Config Register2
CONFIG1     EQU     $001F                   ; Config Register1
TSC         EQU     $0020                   ; TIM Status & Control
TCNTH       EQU     $0021                   ; TIM Counter High
TCNTL       EQU     $0022                   ; TIM Counter Low
TMODH       EQU     $0023                   ; TIM Counter Modulo High
TMODL       EQU     $0024                   ; TIM Counter Modulo Low
TSC0        EQU     $0025                   ; TIM CH0 Status & Control
TCH0H       EQU     $0026                   ; TIM CH0 High
TCH0L       EQU     $0027                   ; TIM CH0 Low
TSC1        EQU     $0028                   ; TIM CH1 Status & Control
TCH1H       EQU     $0029                   ; TIM CH1 High
TCH1L       EQU     $002A                   ; TIM CH1 Low
ADSCR       EQU     $003C                   ; ADC Status & Control
ADR         EQU     $003D                   ; ADC Data Read
ADICLK      EQU     $003E                   ; ADC Input Clock
BSR         EQU     $FE00                   ; Break Status Register
RSR         EQU     $FE01                   ; Reset Status Register
BFCR        EQU     $FE03                   ; Break Flag Control
INT1        EQU     $FE04                   ; Interrupt Status1
INT2        EQU     $FE05                   ; Interrupt Status2
INT3        EQU     $FE06                   ; Interrupt Status3
FLCR        EQU     $FE08                   ; Flash Control Register
FLBPR       EQU     $FE09                   ; Flash Block Protect
BRKH        EQU     $FE0C                   ; Break Address High
BRKL        EQU     $FE0D                   ; Break Address Low
BRKSCR      EQU     $FE0E                   ; Break Status & Control
COPCTL      EQU     $FFFF                   ; Reset COP
;/* End Equate I/O Register of HC08 */;
;
BIT0        EQU     $00                     ; Bit Addressible
BIT1        EQU     $01
BIT2        EQU     $02
BIT3        EQU     $03
BIT4        EQU     $04
BIT5        EQU     $05
BIT6        EQU     $06
BIT7        EQU     $07
;
PERIOD_HI   EQU     $1F                     ; Period = 1mS (8000 x (1/8MHz))
PERIOD_LO   EQU     $40

            ORG     $0080                   ; Strat of RAM
PWM_BUFF    DS      2                       ; PWM Duty Cycle Buffer
PWM_STAS    DS      1                       ; Channel Select
$BASE       10T                             ; Defualt Number to Decimal

            ORG     $EC00                   ; Start Main (68HC908JL3)
            ;
RESET:      BSET    0,CONFIG1               ; Disable COP Watch-Dog
            LDHX    #$0000                  ; Clear PWM Duty Cycle
            STHX    PWM_BUFF
            ;
            LDA     #%00000000              ; X0XXXXXX (Clear TOIE)
            ORA     #%00110000              ; XX11XXXX (Stop & Reset Timer Count)
            ORA     #%00000000              ; XXXXX000 (Prescale = 1)
            STA     TSC
            ;
            LDA     #%01000000              ; X1XXXXXX (CH0IE Enable Interrupt)
            ORA     #%00110000              ; XX11XXXX (Mode Select = Buffer PWM)
            ORA     #%00001000              ; XXXX10XX (Clear Output on Compare)
            ORA     #%00000010              ; XXXXXX1X (Toggle on Overflow)
            STA     TSC0
            ;
            LDHX    PWM_BUFF                ; Define Duty Cycle of PWM
            STHX    TCH0H                   ; TCH0H+TCH0L
            BSET    BIT0,PWM_STAS           ; Next Update CH1
            ;
            MOV     #PERIOD_HI,TMODH        ; Define Period of PWM
            MOV     #PERIOD_LO,TMODL
            ;
            CLI                             ; Enable CPU Interrupt
            BCLR    BIT5,TSC                ; Enable & Strat Timer(TSTOP)
            BRA     $                       ; Loop Here For Wait Interrupt

;/*****************/;
;/* OC0 Interrupt */;
;/*****************/;
;
OC0_SRV:    PSHH                            ; Save Register
            LDA     TSC0                    ; Read CH0 Status
            BCLR    BIT7,TSC0               ; Clear CH0F
            ;
            LDHX    PWM_BUFF                ; Get Duty Cycle
            AIX     #1                      ; Increase 8 Step Duty Cycle
            CPHX    #$1F40                  ; Maximum Duty Cycle
            BLO     NEW_DUTY
            LDHX    #$0000                  ; Restart Duty Cycle
            ;
NEW_DUTY:   STHX    PWM_BUFF                ; Update Duty Cycle
            BRSET   BIT0,PWM_STAS,CH1_EDIT  ; Check Uses Register
            ;
            ;/* Change Duty Cycle CH0 Here */;
            STHX    TCH0H                   ; Define Duty Cycle of PWM
            BSET    BIT0,PWM_STAS           ; Next Update CH1
            BRA     OC0_EXIT
            ;
            ;/* Change Duty Cycle CH1 Here */;
CH1_EDIT:   STHX    TCH1H                   ; Define Duty Cycle of PWM
            BCLR    BIT0,PWM_STAS           ; Next Update CH0
            ;
OC0_EXIT:   PULH                            ; Restore Register
            RTI

            ;/***************/;
            ;/* User Vector */;
            ;/***************/;
            ;
            ORG     $FFF6                   ; TIM CH0 Vector
            DW      OC0_SRV
            ;
            ORG     $FFFE                   ; Reset Vector
            DW      RESET

            END

